CON
  MaxOutput = 450
  MaxIntegral = $0800_0000  

VAR
  long Kp             'PID Gain
  long Ki             'PID Gain
  long Kd             'PID Gain
  long Kd2            'PID Gain
  long PMax           'Maximum P term error value
  
  long Output         'PID Output
  long PError, PClamped
  long DError, D2Error
  long IError         'Accumulated integral error
  long LastPError     'Previous Error
  long LastDError     'Previous Error

   
PUB Init( PGain, IGain, DGain )
  Kp := PGain
  Ki := IGain
  Kd := DGain
  Kd2 := 0

  LastPError := 0
  IError := 0


PUB SetPGain( Value )
  Kp := Value  

PUB SetIGain( Value )
  Ki := Value  

PUB SetDGain( Value )
  Kd := Value

PUB SetD2Gain( Value )
  Kd2 := Value

PUB SetPIDGain( PG, IG, DG )
  Kp := PG  
  Ki := IG  
  Kd := DG
  

PUB SetPMax( Value )
  PMax := Value


PUB ResetIntegralError
  IError := 0

PUB GetVarAddress
  return @Output
  
   
PUB Calculate( SetPoint , Measured , DoIntegrate )

  ' Proportional error is Desired - Measured
  PError := SetPoint - Measured
  
  ' Derivative error is the delta PError divided by time
  ' If loop timing is const, you can skip the divide and just make the factor smaller
  DError := PError - LastPError
  D2Error := DError - LastDError  

  LastDError := DError
  LastPError := PError

  PClamped := PError
  if( PMax > 0 )
    PClamped #>= -PMax
    PClamped <#= PMax

  Output := (Kp ** PClamped) + (Kd ** DError) + (Kd2 ** D2Error) + (Ki ** IError)
  
  'Accumulate Integral error *or* Limit output. 
  'Stop accumulating when output saturates 

  Output := -MaxOutput #> Output <# MaxOutput
     
  if( DoIntegrate )
    IError += PError
    IError := -MaxIntegral #> IError <# MaxIntegral  
     
  return Output


PUB Calculate_ForceD( SetPoint , Measured , Deriv , DoIntegrate )

  ' Proportional error is Desired - Measured
  PError := SetPoint - Measured
  
  ' Derivative error is the delta PError divided by time
  ' If loop timing is const, you can skip the divide and just make the factor smaller
  DError := Deriv
  D2Error := DError - LastDError  

  LastDError := DError
  LastPError := PError

  Output := (Kp ** PError) + (Kd ** DError) + (Kd2 ** D2Error) + (Ki ** IError)
  
  'Accumulate Integral error *or* Limit output. 
  'Stop accumulating when output saturates 
     
  Output := -MaxOutput #> Output <# MaxOutput
     
  if( DoIntegrate )
    IError += PError
    IError := -MaxIntegral #> IError <# MaxIntegral  
     
  return Output


PUB Calculate_ForceD_NoD2( SetPoint , Measured , Deriv , DoIntegrate )

  ' Proportional error is Desired - Measured
  PError := SetPoint - Measured
  
  ' Derivative error is the delta PError divided by time
  ' If loop timing is const, you can skip the divide and just make the factor smaller
  DError := Deriv

  LastDError := DError
  LastPError := PError

  Output := (Kp ** PError) + (Kd ** DError) + (Ki ** IError)
  
  'Accumulate Integral error *or* Limit output. 
  'Stop accumulating when output saturates 
     
  Output := -MaxOutput #> Output <# MaxOutput
     
  if( DoIntegrate )
    IError += PError
    IError := -MaxIntegral #> IError <# MaxIntegral  
     
  return Output
   